@@ -226,7 +226,8 @@ def _dotfile_install(o, dotfile, tmpdir=None):
                               actionexec=pre_actions_exec,
                               is_template=is_template,
                               ignore=ignores,
-                              chmod=dotfile.chmod)
+                              chmod=dotfile.chmod,
+                              force_chmod=o.install_force_chmod)
     elif hasattr(dotfile, 'link') and \
             dotfile.link == LinkTypes.LINK_CHILDREN:
         # link_children
@@ -235,7 +236,8 @@ def _dotfile_install(o, dotfile, tmpdir=None):
                               actionexec=pre_actions_exec,
                               is_template=is_template,
                               chmod=dotfile.chmod,
-                              ignore=ignores)
+                              ignore=ignores,
+                              force_chmod=o.install_force_chmod)
     else:
         # nolink
         src = dotfile.src
@@ -256,7 +258,8 @@ def _dotfile_install(o, dotfile, tmpdir=None):
                               noempty=dotfile.noempty,
                               ignore=ignores,
                               is_template=is_template,
-                              chmod=dotfile.chmod)
+                              chmod=dotfile.chmod,
+                              force_chmod=o.install_force_chmod)
         if tmp:
             tmp = os.path.join(o.dotpath, tmp)
             if os.path.exists(tmp):
@@ -643,7 +646,7 @@ def cmd_remove(o):
                         break
                     removepath(parent, LOG)
                 parent = os.path.dirname(parent)
-            removed.append(dotfile.key)
+            removed.append(dotfile)
 
     if o.dry:
         LOG.dry('new config file would be:')
@@ -651,7 +654,10 @@ def cmd_remove(o):
     else:
         o.conf.save()
     if removed:
-        LOG.log('\ndotfile(s) removed: {}'.format(','.join(removed)))
+        LOG.log('\nFollowing dotfile(s) are not tracked anymore:')
+        entries = ['- \"{}\" (was tracked as \"{}\")'.format(r.dst, r.key)
+                   for r in removed]
+        LOG.log('\n'.join(entries))
     else:
         LOG.log('\nno dotfile removed')
     return True
